@@ -1,22 +1,36 @@
-"""Flask-Scrypt flask extension provides scrypt password hashing and random salt generation. Hashes and Salts are base64 encoded.
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+
 """
+Flask-Scrypt flask extension provides scrypt password hashing and random salt generation.
+Hashes and Salts are base64 encoded.
+"""
+from __future__ import print_function, unicode_literals
 import sys
 import base64
+import hmac
 from os import urandom
 
-__version_info__ = ('0', '1', '3', '1')
+try:
+    from itertools import izip
+except ImportError:
+    izip = zip
+
+_builtin_safe_str_cmp = getattr(hmac, 'compare_digest', None)
+
+__version_info__ = ('0', '1', '3', '6')
 __version__ = '.'.join(__version_info__)
 __author__ = 'Gilbert Robinson'
 __license__ = 'MIT'
-__copyright__ = 'Copyright (c) 2013 Gilbert Robinson'
-__all__ = ['generate_password_hash', 'generate_random_salt', \
-        'check_password_hash', 'enbase64', 'debase64']
+__copyright__ = 'Copyright (c) 2013 Gilbert Robinson, Copyright (c) 2014 Samuel Marks'
+__all__ = ['generate_password_hash', 'generate_random_salt',
+           'check_password_hash', 'enbase64', 'debase64']
 
 try:
-    import scrypt
+    from scrypt import hash as scrypt_hash
 except ImportError as err:
     print('Please install py-scrypt package. Error: ', err)
-    raise(err)
+    raise err
 
 PYTHON2 = sys.version_info < (3, 0)
 
@@ -79,8 +93,8 @@ def generate_password_hash(password, salt, N=1 << 14, r=8, p=1, buflen=64):
     if PYTHON2:
         password = password.encode('utf-8')
         salt = salt.encode('utf-8')
-    scrypt_hash = scrypt.hash(password, salt, N, r, p, buflen)
-    return enbase64(scrypt_hash)
+    pw_hash = scrypt_hash(password, salt, N, r, p, buflen)
+    return enbase64(pw_hash)
 
 
 def generate_random_salt(byte_size=64):
@@ -96,7 +110,7 @@ def generate_random_salt(byte_size=64):
     return enbase64(urandom(byte_size))
 
 
-def check_password_hash(password, password_hash, salt):
+def check_password_hash(password, password_hash, salt, N=1 << 14, r=8, p=1, buflen=64):
     """
     Given a password, hash, salt this function verifies the password is equal to hash/salt.
 
@@ -106,11 +120,28 @@ def check_password_hash(password, password_hash, salt):
     Returns:
        - ``bool``
     """
-    if PYTHON2:
-        password = password.encode('utf-8')
-        password_hash = password_hash.encode('utf-8')
-        salt = salt.encode('utf-8')
+    candidate_hash = generate_password_hash(password, salt, N, r, p, buflen)
 
-    scrypt_hash = generate_password_hash(password, salt)
+    return safe_str_cmp(password_hash, candidate_hash)
 
-    return password_hash == scrypt_hash
+
+def safe_str_cmp(a, b):
+    """This function compares strings in somewhat constant time. This
+    requires that the length of at least one string is known in advance.
+
+    Returns `True` if the two strings are equal, or `False` if they are not.
+
+    From: https://github.com/mitsuhiko/werkzeug/blob/master/werkzeug/security.py
+    """
+    if _builtin_safe_str_cmp is not None:
+        return _builtin_safe_str_cmp(a, b)
+    elif len(a) != len(b):
+        return False
+    rv = 0
+    if not PYTHON2 and isinstance(a, bytes) and isinstance(b, bytes):
+        for x, y in izip(a, b):
+            rv |= x ^ y
+    else:
+        for x, y in izip(a, b):
+            rv |= ord(x) ^ ord(y)
+    return rv == 0
